==> org-lsst-ccs-drivers-ascii-1.0.1-sources.jar: org/lsst/ccs/drivers/ascii/AsciiIOFtdi.java <==
package org.lsst.ccs.drivers.ascii;

import org.lsst.ccs.drivers.commons.DriverException;
import org.lsst.ccs.drivers.commons.DriverTimeoutException;
import org.lsst.ccs.drivers.ftdi.Ftdi;

/**
 *****************************************************************************
 **
 **  Communications I/O routines for serial devices connected via FTDI.
 **
 **  @author Owen Saxton
 **
 *****************************************************************************
 */

public class AsciiIOFtdi implements AsciiIO {

   /**
    **************************************************************************
    **
    **  Private fields.
    **
    **************************************************************************
    */
    Ftdi ftdi = new Ftdi();


   /**
    **************************************************************************
    **
    **  Opens a connection.
    **
    **  @param  serial  The serial number string of the interface
    **
    **  @param  baud    The baud rate
    **
    **  @param  index   The index of the device
    **
    **  @throws  DriverException
    **
    **************************************************************************
    */
    @Override
    public void open(String serial, int baud, int index) throws DriverException
    {
        ftdi.open(index, serial);
        ftdi.setBaudrate(baud);
    }


   /**
    **************************************************************************
    **
    **  Closes the connection.
    **
    **  @throws  DriverException
    **
    **************************************************************************
    */
    @Override
    public void close() throws DriverException
    {
        ftdi.close();
    }


   /**
    **************************************************************************
    **
    **  Writes a command.
    **
    **  @param  command  The command to write, including the terminator
    **
    **  @throws  DriverException
    **
    **************************************************************************
    */
    @Override
    public void write(byte[] command) throws DriverException
    {
        ftdi.write(command);
    }


   /**
    **************************************************************************
    **
    **  Reads available response data.
    **
    **  @param  buff    The buffer to receive the response data
    **
    **  @param  offset  The offset to the first available byte in the buffer
    **
    **  @return  The number of bytes read
    **
    **  @throws  DriverException
    **  @throws  DriverTimeoutException
    **
    **************************************************************************
    */
    @Override
    public int read(byte[] buff, int offset) throws DriverException
    {
        int leng = 0, count = ftdi.getQueueStatus();
        if (count == 0) {
            leng = ftdi.read(buff, offset, 1);
            if (leng == 0) {
                throw new DriverTimeoutException("Read timed out");
            }
            offset++;
            count = ftdi.getQueueStatus();
        }
        if (offset + count > buff.length) {
            count = buff.length - offset;
        }
        if (count > 0) {
            leng += ftdi.read(buff, offset, count);
        }
        return leng;
    }


   /**
    **************************************************************************
    **
    **  Flushes any available response data.
    **
    **  @throws  DriverException
    **
    **************************************************************************
    */
    @Override
    public void flush() throws DriverException
    {
        int count = ftdi.getQueueStatus();
        if (count > 0) {
            byte[] buff = new byte[count];
            ftdi.read(buff, 0, count);
        }
    }

    
   /**
    **************************************************************************
    **
    **  Sets the receive timeout.
    **
    **  @param  timeout  The receive timeout (ms).  0 means no timeout.
    **
    **  @throws  DriverException
    **
    **************************************************************************
    */
    @Override
    public void setTimeout(int timeout) throws DriverException
    {
        ftdi.setTimeouts(timeout, 0);
    }

}

==> org-lsst-ccs-drivers-ascii-1.1.1-sources.jar: org/lsst/ccs/drivers/ascii/AsciiIOFtdi.java <==
package org.lsst.ccs.drivers.ascii;

import java.util.HashMap;
import java.util.Map;
import org.lsst.ccs.drivers.commons.DriverException;
import org.lsst.ccs.drivers.commons.DriverTimeoutException;
import org.lsst.ccs.drivers.ftdi.Ftdi;

/**
 *****************************************************************************
 **
 **  Communications I/O routines for serial devices connected via FTDI.
 **
 **  @author Owen Saxton
 **
 *****************************************************************************
 */

public class AsciiIOFtdi implements AsciiIO {

   /**
    **************************************************************************
    **
    **  Private fields.
    **
    **************************************************************************
    */
    private final static Map<Integer, Integer> dbitsMap = new HashMap<>();
    static {
        dbitsMap.put(Ascii.DBITS_SEVEN, Ftdi.DATABITS_7);
        dbitsMap.put(Ascii.DBITS_EIGHT, Ftdi.DATABITS_8);
    }

    private final static Map<Integer, Integer> sbitsMap = new HashMap<>();
    static {
        sbitsMap.put(Ascii.SBITS_ONE, Ftdi.STOPBITS_1);
        sbitsMap.put(Ascii.SBITS_TWO, Ftdi.STOPBITS_2);
    }

    private final static Map<Integer, Integer> parityMap = new HashMap<>();
    static {
        parityMap.put(Ascii.PARITY_NONE,  Ftdi.PARITY_NONE);
        parityMap.put(Ascii.PARITY_ODD,   Ftdi.PARITY_ODD);
        parityMap.put(Ascii.PARITY_EVEN,  Ftdi.PARITY_EVEN);
        parityMap.put(Ascii.PARITY_MARK,  Ftdi.PARITY_MARK);
        parityMap.put(Ascii.PARITY_SPACE, Ftdi.PARITY_SPACE);
    }

    Ftdi ftdi = new Ftdi();
    private int timeout = 0;


   /**
    **************************************************************************
    **
    **  Opens a connection.
    **
    **  @param  serial  The serial number string of the interface
    **
    **  @param  baud    The baud rate
    **
    **  @param  dchars  The data characteristics
    **
    **  @throws  DriverException
    **
    **************************************************************************
    */
    @Override
    public void open(String serial, int baud, int dchars) throws DriverException
    {
        Integer dataBits = dbitsMap.get((dchars >> 24) & 0xff);
        if (dataBits == null) {
            throw new DriverException("Invalid number of data bits");
        }
        Integer stopBits = sbitsMap.get((dchars >> 16) & 0xff);
        if (stopBits == null) {
            throw new DriverException("Invalid number of stop bits");
        }
        Integer parity = parityMap.get((dchars >> 8) & 0xff);
        if (parity == null) {
            throw new DriverException("Invalid parity");
        }
        int fc = dchars & 0xff;
        int flowCtrl =
          ((fc & Ascii.FLOW_RTS_CTS) != 0 ? Ftdi.FLOW_CONTROL_RTS_CTS : 0)
           | ((fc & Ascii.FLOW_DTR_DSR) != 0 ? Ftdi.FLOW_CONTROL_DTR_DSR : 0)
           | ((fc & Ascii.FLOW_XON_XOFF) != 0 ? Ftdi.FLOW_CONTROL_XON_XOFF : 0);
        ftdi.open(0, serial);
        ftdi.setBaudrate(baud);
        ftdi.setDataCharacteristics(dataBits, stopBits, parity);
        ftdi.setFlowControl(flowCtrl);
        ftdi.enableEvents(Ftdi.EVENT_RXCHAR);
    }


   /**
    **************************************************************************
    **
    **  Closes the connection.
    **
    **  @throws  DriverException
    **
    **************************************************************************
    */
    @Override
    public void close() throws DriverException
    {
        ftdi.close();
    }


   /**
    **************************************************************************
    **
    **  Writes a command.
    **
    **  @param  command  The command to write, including the terminator
    **
    **  @throws  DriverException
    **
    **************************************************************************
    */
    @Override
    public void write(byte[] command) throws DriverException
    {
        ftdi.write(command);
    }


   /**
    **************************************************************************
    **
    **  Reads available response data.
    **
    **  @param  buff    The buffer to receive the response data
    **
    **  @param  offset  The offset to the first available byte in the buffer
    **
    **  @return  The number of bytes read
    **
    **  @throws  DriverException
    **  @throws  DriverTimeoutException
    **
    **************************************************************************
    */
    @Override
    public int read(byte[] buff, int offset) throws DriverException
    {
        int leng = ftdi.getQueueStatus();
        if (leng == 0) {
            if (ftdi.awaitEvent(timeout) == 0) {
                throw new DriverTimeoutException("Read timed out");
            }
            leng = ftdi.getQueueStatus();
        }
        if (offset + leng > buff.length) {
            leng = buff.length - offset;
        }
        if (leng > 0) {
            leng = ftdi.read(buff, offset, leng);
        }

        return leng;
    }


   /**
    **************************************************************************
    **
    **  Flushes any available response data.
    **
    **  @throws  DriverException
    **
    **************************************************************************
    */
    @Override
    public void flush() throws DriverException
    {
        int count = ftdi.getQueueStatus();
        if (count > 0) {
            byte[] buff = new byte[count];
            ftdi.read(buff, 0, count);
        }
    }

    
   /**
    **************************************************************************
    **
    **  Sets the receive timeout.
    **
    **  @param  timeout  The receive timeout (ms).  0 means no timeout.
    **
    **  @throws  DriverException
    **
    **************************************************************************
    */
    @Override
    public void setTimeout(int timeout) throws DriverException
    {
        this.timeout = timeout;
    }

}
